--[[

Author : Levva of EU Khadgar 

Version 0.01
	Initial release of Concept code to wowace svn

Version 0.10
	Inplemented quest item bag frame
	Now searches bags and displays icons in quest bag frame 
	There is an issue with the tooltip displaying a bag tooltip rather than an item tooltip 
	Clicking the buttons does absolutely nothing at present
	
Version 0.11 
	Add a /qi open command to reopen a closed bag 
	Test fix for issue where bag with multiple items slot wasnt clearing text
	Test fix for removing items from bag was leaving old slot icons
	
Version 0.20
	Fixed removing an item resizing bag and clearing old slot icons
	Fixed taking new quest actually updates bag
	Finally got the tooltips working and in doing so the default interface takes care of using items etc etc 
		so it acts just like a regular bag with minimal coding - yay!!
		
Version 0.30
	Force update of bagslot parent to remove problem of stuff being moved in bag getting out of sync
	Added ITEM_PUSH event to catch issue of sometimes not firing bag update if item goes in backpack from looting
	
Version 1.00
	Added key binding option to open bag frame
	Removed extra empty row if exact multiple of 4 items in bag
	Tidied up un-needed extra code
	Added portrait icon for bag
	Remember state of bag window when relogging
	
Version 1.01 
	Remove some debug info
	
Version 1.10
	Added Item cooldown displays for quest items that have a cooldown on use
	
Version 1.20
	Added include & exclude options to override what the Blizzard item type is 
		and thus force inclusion or exclusion from quest item bag
	Added Scale option so users can scale the size of the bag.
	Added /qi include and /qi exclude code - doesn't do much yet plan is for user lists of includes & excludes
	
Version 1.21 
	Hide open config panel in config panel
	
--]]

local L = LibStub("AceLocale-3.0"):GetLocale("QuestItem")
local AceAddon = LibStub("AceAddon-3.0")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
QuestItem = AceAddon:NewAddon("QuestItem", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
local REVISION = tonumber(("$Revision: 27 $"):match("%d+"))

QuestItem.textures = {}
QuestItem.borders = {}
QuestItem.fonts = {}
QuestItem.sounds = {}

-- Binding Variables
BINDING_HEADER_QUESTITEM_TITLE = L["Keybind Title"]
BINDING_NAME_QUESTITEM_OPEN_BAG = L["Open Quest Bag"]

-------------------
-- Config details
-------------------

QuestItem.defaults = {
	char = {
		message = "Welcome Home!",
		relativeTo = "UIParent",
		relativePoint = "CENTER",
		point = "CENTER",
		texture = "Blizzard",
		msgfont = "Friz Quadrata TT",
		msgfontsize = 24,
		msgfonteffect = "none",
		border = "None",
		barborder = "Blizzard Tooltip",
		xOffset = 0,
		yOffset = 0,
		fWidth = 300,
		fHeight = 175,
		scale = 1,
		debug = false,
		shown = true,
	}
}

-----------------------------------------
-- Initialisation & Startup Routines
-----------------------------------------

function QuestItem:OnInitialize()
	local AceConfigReg = LibStub("AceConfigRegistry-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")

	self.db = LibStub("AceDB-3.0"):New("QuestItemDBPC", QuestItem.defaults, "char")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("QuestItem", self:GetOptions(), {"questitem", "qi"} )
	media.RegisterCallback(self, "LibSharedMedia_Registered")
	
	-- Add the options to blizzard frame (add them backwards so they show up in the proper order
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("QuestItem","QuestItem")
	self.optionsFrame[L["About"]] = LibStub("LibAboutPanel").new("QuestItem", "QuestItem")
	self.db:RegisterDefaults(self.defaults)
	
	self:CreateBagFrame()
	local version = GetAddOnMetadata("QuestItem","Version")
	self.version = ("QuestItem v%s (r%s)"):format(version, REVISION)
	self:Print(self.version.." Loaded.")
end

function QuestItem:OnDisable()
    -- Called when the addon is disabled
end

function QuestItem:OnEnable()
	self:LibSharedMedia_Registered()
 	self:RegisterEvent("PLAYER_ENTERING_WORLD")
end

function QuestItem:LibSharedMedia_Registered()
	for k, v in pairs(media:List("statusbar")) do
		self.textures[v] = v
	end
	for k, v in pairs(media:List("border")) do
		self.borders[v] = v
	end
	for k, v in pairs(media:List("font")) do
		self.fonts[v] = v
	end
	for k, v in pairs(media:List("sound")) do
		self.sounds[v] = v
	end
end

----------------------
-- Event Routines
----------------------

function QuestItem:PLAYER_ENTERING_WORLD()
	if self.db.char.shown then
		self:DisplayQuestBag()
		if self.db.char.debug then
			self:Print("shown is true")
		end
	else
		self:QuestBag_OnHide()
		if self.db.char.debug then
			self:Print("shown is false")
		end
	end
end

function QuestItem:BAG_UPDATE_COOLDOWN()
	if self.db.char.debug then
		self:Print("bag update cooldown fired")
	end
	for index = 1, self.db.char.questbag.count do
		local frameName = self.BagFrame:GetName()
		local buttonName = frameName.."Item"..index
		local itemButton = getglobal(buttonName)
		local bagID = itemButton:GetParent():GetID()
		if GetContainerItemInfo(bagID, itemButton:GetID()) then
			ContainerFrame_UpdateCooldown(bagID, itemButton);
		else
			getglobal(buttonName.."Cooldown"):Hide();
		end
	end
end

function QuestItem:BAG_UPDATE()
	if self.db.char.debug then
		self:Print("bag update fired for "..arg1)
	end
	if arg1 and arg1 >= 0 and arg1 <= 4 then -- fire for carried bags only
		self:UpdateBagFrame()
	end
end

function QuestItem:ITEM_PUSH()
	if self.db.char.debug then
		self:Print("item push fired for "..arg1)
	end
	if arg1 and arg1 >= 0 and arg1 <= 4 then -- fire for carried bags only
		self:UpdateBagFrame()
	end
end

function QuestItem:QuestBag_OnHide()
	self:UnregisterEvent("BAG_UPDATE")
	self:UnregisterEvent("ITEM_PUSH")
	self:UnregisterEvent("BAG_UPDATE_COOLDOWN")
	self.BagFrame:Hide()
	self.db.char.shown = false
end

----------------------
-- Bag Routines
----------------------

function QuestItem:DisplayQuestBag()
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("ITEM_PUSH")
	self:RegisterEvent("BAG_UPDATE_COOLDOWN")
	self:UpdateBagFrame()
	self.BagFrame:Show()
	self.db.char.shown = true
end

function QuestItem:UpdateBagFrame()
	-- check all items in bags and add to quest bag frame 
	self:ClearQuestBag()
	for bag = 0,4 do
		for slot = 1, GetContainerNumSlots(bag) do
			local itemLink = GetContainerItemLink(bag, slot)
			if itemLink then -- check if its a quest item
				local itemName, itemString, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink)
				if itemString then
					local _, itemID = strsplit(":", itemString)
					if (itemType == L["Quest"] and not self:IsExcluded(itemID)) or self:IsIncluded(itemID) then
						self:AddItemtoQuestBag(bag, slot, itemLink)
					end
				end
			end
		end
	end
	self:RedrawBagFrame()
end

function QuestItem:ClearQuestBag()
	if self.db.char.questbag then
		if not self.db.char.questbag.rows then
			self.db.char.questbag.rows = 1
		end
		local buttonCount = 4 * self.db.char.questbag.rows
		for i = 1, buttonCount do 
			local name = self.BagFrame:GetName()
			local buttonName = name.."Item"..i
			local itemButton = getglobal(buttonName)
			if itemButton then
				itemButton:Hide()
			end
		end
	end
	self.db.char.questbag = {}
	self.db.char.questbag.count = 0
	self.db.char.questbag.rows = 1 -- always have one row even if no items
end

function QuestItem:AddItemtoQuestBag(bag, slot, itemLink)
	-- this will store the quest items in the bag frame
	local index = self.db.char.questbag.count + 1
	self.db.char.questbag.count = index
	self.db.char.questbag.rows = math.floor((index - 1) / 4) + 1
	self.db.char.questbag[index] = {}
	self.db.char.questbag[index].bag = bag 
	self.db.char.questbag[index].slot = slot
	self.db.char.questbag[index].itemLink = itemLink 
end